# *****************************************************************
#
# Licensed Materials - Property of IBM
#
# (C) Copyright IBM Corp. 2019 . All Rights Reserved.
#
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
#
# *****************************************************************

import numpy as np
import pandas as pd
from sklearn.neighbors import NearestNeighbors as skKNN
from pai4sk.neighbors import NearestNeighbors as pai4skKNN
import cudf
import os
import time
import argparse

CLI=argparse.ArgumentParser()
CLI.add_argument(
    "--nrows",
    type=int,
    default=2**16
)
CLI.add_argument(
    "--ncols",
    type=int,
    default=40
)
CLI.add_argument(
    "--n_neighbors",
    type=int,
    default=2
)
CLI.add_argument(
    "--skip_sklearn",
    action = 'store_true',
    default=False
)
args = CLI.parse_args()

import gzip
def load_data(nrows, ncols, cached = 'data/mortgage.npy.gz',source='mortgage'):
    if os.path.exists(cached) and source=='mortgage':
        print('use mortgage data')
        with gzip.open(cached) as f:
            X = np.load(f)
        X = X[np.random.randint(0,X.shape[0]-1,nrows),:ncols]
    else:
        print('use random data')
        X = np.random.random((nrows,ncols)).astype('float32')
    df = pd.DataFrame({'fea%d'%i:X[:,i] for i in range(X.shape[1])}).fillna(0)
    return df

from sklearn.metrics import mean_squared_error
def array_equal(a,b,threshold=1e-1,with_sign=True,metric='mse'):
    a = to_nparray(a)
    b = to_nparray(b)
    if with_sign == False:
        a,b = np.abs(a),np.abs(b)
    if metric=='mse':
        error = mean_squared_error(a,b)
    else:
        error = np.sum(a!=b)/(a.shape[0]*a.shape[1])
    res = error<threshold
    return res

def to_nparray(x):
    if isinstance(x,np.ndarray) or isinstance(x,pd.DataFrame):
        return np.array(x)
    elif isinstance(x,np.float64):
        return np.array([x])
    elif isinstance(x,cudf.DataFrame) or isinstance(x,cudf.Series):
        return x.to_pandas().values
    return x

nrows = args.nrows
ncols = args.ncols
n_neighbors = args.n_neighbors

X1 = load_data(nrows, ncols)
print('data', X1.shape)

X = cudf.DataFrame.from_pandas(X1)
t0 = time.time()
knn_pai4sk = pai4skKNN()
knn_pai4sk.fit(X)
D_pai4sk,I_pai4sk = knn_pai4sk.kneighbors(X, n_neighbors)
print("[pai4sk] time (s):  {0:.2f}".format(time.time()-t0))

if args.skip_sklearn:
    print('Execution using scikit-learn is skipped.')
    exit()

t0 = time.time()
knn_sk = skKNN()
knn_sk.fit(X1)
D_sk,I_sk = knn_sk.kneighbors(X1, n_neighbors)
print("[sklearn] time (s):  {0:.2f}".format(time.time()-t0))

passed = array_equal(D_sk, D_pai4sk)
message = 'compare knn: pai4sk vs sklearn distances %s'%('equal'if passed else 'NOT equal')
print(message)
passed = array_equal(I_sk, I_pai4sk)
message = 'compare knn: pai4sk vs sklearn indexes %s'%('equal'if passed else 'NOT equal')
print(message)
